<?php
/**
 * @file
 * Query string process class.
 */

class FunnelbackQueryString {

  /**
   * Get facet query string from raw query string.
   *
   * @param array $raw_queries
   * @return array
   */
  public static function funnelbackFilterFacetQueryString($raw_queries = []) {
    $facet_query = [];
    foreach ($raw_queries as $param) {
      if (substr($param, 0, 2) == 'f.') {
        // Compose query string array.
        $whole_string = explode('=', $param);
        $facet_query[$whole_string[0]][] = $whole_string[1];
      }
    }

    return $facet_query;
  }

  /**
   * Get facet query string from raw query string.
   *
   * @param array $raw_queries
   * @return array
   */
  public static function funnelbackFilterContextualQueryString($raw_queries = []) {
    $contextual_query = [];
    foreach ($raw_queries as $param) {
      if (substr($param, 0, 7) == 'cluster' ||
        substr($param, 0, 15) == 'clicked_fluster') {
        // Compose query string array.
        $whole_string = explode('=', $param);
        $contextual_query[$whole_string[0]]= $whole_string[1];
      }
    }

    return $contextual_query;
  }

  /**
   * Remove system default query strings from link.
   *
   * @param $query_string
   * @return string
   */
  public static function funnelbackFilterSystemQueryString($query_string) {
    $query_string = str_replace('?', '', $query_string);
    $strings = explode('&', $query_string);
    foreach ($strings as $key => $string) {
      if (substr($string, 0, 10) == 'remote_ip='||
        substr($string, 0, 8) == 'profile='||
        substr($string, 0, 11) == 'collection=') {
        // Remove system query strings.
        unset($strings[$key]);
      }
    }

    return '?'. implode('&', $strings);
  }

  /**
   * Normalise query.
   *
   * @param $string
   * @return mixed|string
   */
  public static function funnelbackQueryNormaliser($string) {
    $string = str_replace("`", '', $string);
    $string = str_replace(' ', '+', $string);

    return $string;
  }
}