<?php

require_once __DIR__ . '/../../vendor/e0ipso/drupal-unit-autoload/autoload.php';
require_once __DIR__ . '/../../../includes/Funnelback.class.php';
require_once __DIR__ . '/../../../includes/FunnelbackRequest.class.php';
require_once __DIR__ . '/../../../includes/FunnelbackQueryString.class.php';

use PHPUnit\Framework\TestCase;

class FunnelbackTestCase extends TestCase {

  protected $successResponse;

  protected $suggestResponse;

  public function setUp() {
    parent::setUp();

    $this->successResponse = (object) [
      'request' => NULL,
      'data' => file_get_contents(dirname(__FILE__) . '/../../mocks/response.json', FILE_USE_INCLUDE_PATH),
      'protocol' => 'HTTP/1.1',
      'status_message' => 'OK',
      'headers' => [],
      'code' => 200,
    ];

    $this->suggestResponse = (object) [
      'request' => NULL,
      'data' => file_get_contents(dirname(__FILE__) . '/../../mocks/suggestresponse.json', FILE_USE_INCLUDE_PATH),
      'protocol' => 'HTTP/1.1',
      'status_message' => 'OK',
      'headers' => [],
      'code' => 200,
    ];
  }

  /**
   * Test funnelback_json_query function.
   */
  public function testDoQuery() {
    $funnelbackRequest = Mockery::mock('FunnelbackRequest');
    $funnelbackRequest
      ->shouldReceive('funnelbackRequest')
      ->andReturn($this->successResponse);


    $funnelbackRequest
      ->shouldReceive('funnelbackDebug')
      ->andReturnNull();
    $funnelback = new Funnelback(NULL,NULL, NULL, NULL);

    $response = $funnelback->funnelbackJsonQuery(json_decode($funnelbackRequest->funnelbackRequest(NULL, NULL, NULL)->data, TRUE), '');

    $this->assertEquals($this->getJsonQueryResult(), $response['results']);
  }

  /**
   * Test autocomplete response.
   */
  public function testSuggestQuery() {
    $funnelbackRequest = Mockery::mock('FunnelbackRequest');
    $funnelbackRequest
      ->shouldReceive('funnelbackRequest')
      ->andReturn($this->suggestResponse);


    $funnelbackRequest
      ->shouldReceive('funnelbackDebug')
      ->andReturnNull();
    $funnelback = new Funnelback(NULL,NULL, NULL, NULL);

    $response = $funnelback->funnelbackJsonQuery(json_decode($funnelbackRequest->funnelbackRequest(NULL, NULL, NULL)->data, TRUE), '');

    $this->assertEquals($this->getSuggestJsonQueryResult(), $response);
  }

  /**
   * Test filter facet query string function.
   */
  public function testFilterFacetQueryString() {
    $funnelback = new FunnelbackQueryString();
    $raw_strings = [
      'query=holden',
      'f.Condition%7Ccondition=demo',
      'f.Extras%7Extras=aircondition',
      'f.Extras%7Extras=cruise+control',
    ];

    $facet_strings = [
      'f.Condition%7Ccondition' => [
        'demo',
      ],
      'f.Extras%7Extras' => [
        'aircondition',
        'cruise+control',
      ]
    ];

    $this->assertEquals($facet_strings, $funnelback->funnelbackFilterFacetQueryString($raw_strings));
  }

  /**
   * Test filter system query string function.
   */
  public function testFilterSystemQueryString() {
    $funnelback = new FunnelbackQueryString();
    $testQueryString = '?remote_ip=10.20.1.1&query=holden&profile=_default_preview&f.Condition%7CAllDocumentsFill=All&collection=fb-bargain-store-meta';
    $fiteredQueryString = '?query=holden&f.Condition%7CAllDocumentsFill=All';

    $this->assertEquals($fiteredQueryString, $funnelback->funnelbackFilterSystemQueryString($testQueryString));
  }

  /**
   * Test filter contextual query string function.
   */
  public function testContextualFilter() {
    $funnelback = new FunnelbackQueryString();
    $raw_strings = [
      'cluster1=my+holden',
      'cluster0=holden',
      'start_rank=0',
      'query=About+my+holden',
      'cluster2=about+holden'
    ];

    $contextual_strings = [
      'cluster1' => 'my+holden',
      'cluster0' => 'holden',
      'cluster2' => 'about+holden',
    ];

    $this->assertEquals($contextual_strings, $funnelback->funnelbackFilterContextualQueryString($raw_strings));
  }

  /**
   * Test query normaliser.
   */
  public function testQueryNormaliser() {
    $funnelback = new FunnelbackQueryString();
    $raw_string = "`my holden is awesome`";
    $expected_string = 'my+holden+is+awesome';

    $this->assertEquals($expected_string, $funnelback->funnelbackQueryNormaliser($raw_string));
  }

  protected function getJsonQueryResult() {
    return [
      [
        'title' => 'https://sample.data.io/Holden/Demo/Blue/2',
        'date' => '1485781200000',
        'summary' => NULL,
        'live_url' => 'sample.data.io/Holden/Demo/Blue/2',
        'cache_url' => '/s/cache?collection=fb-cars-xml&doc=funnelback-web-crawl.warc&off=408&len=353&url=https%3A%2F%2Fsample.data.io%2FHolden%2FDemo%2FBlue%2F2&profile=_default_preview',
        'display_url' => 'https://sample.data.io/Holden/Demo/Blue/2',
        'metaData' => [
          'condition' => 'Demo',
          'colour' => 'Blue',
          'd' => '31 January 2017',
          'price' => '25000',
          'extras' => 'Aircondition, Power Steering, Electric Windows, Cruise Control',
          'make' => 'Holden',
          'nodeId' => 1,
        ],
      ]
    ];
  }

  protected function getSuggestJsonQueryResult() {
    return [
      "hdmi" => "hdmi",
      "hp" => "hp",
      "holden" => "holden"
    ];
  }
}
